#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Setup file for pypixplore.

    This file was generated with PyScaffold 2.5.7, a tool that easily
    puts up a scaffold for your new Python project. Learn more under:
    http://pyscaffold.readthedocs.org/
"""

import sys
from setuptools import setup
from pypixplore import __version__

with open('README.rst') as f:
    long_desc = f.read()


def setup_package():
    needs_sphinx = {'build_sphinx', 'upload_docs'}.intersection(sys.argv)
    sphinx = ['sphinx'] if needs_sphinx else []
    setup(
        setup_requires=['six', 'pyscaffold>=2.5a0,<2.6a0'] + sphinx,
        use_pyscaffold=True,
        version=__version__,
        long_description=long_desc,
        install_requires=['pip', 'tinydb', 'pipdeptree'],
    )


if __name__ == "__main__":
    setup_package()
